@@ -25,6 +25,8 @@ using System.Xml.Xsl;
 using System.Text;
 using System.Collections;
 using ICSharpCode.SharpZipLib.Zip;
+using Monodoc.Lucene.Net.Index;
+using Monodoc.Lucene.Net.Documents;
 
 using BF = System.Reflection.BindingFlags;
 
@@ -478,6 +480,21 @@ public class EcmaHelpSource : HelpSource {
 		ExtObject = new ExtensionObject (this);
 	}
 
+	static string css_ecma;
+	public static string css_ecma_code {
+		get {
+			if (css_ecma != null)
+				return css_ecma;
+			if (use_css) {
+				System.Reflection.Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
+				Stream str_css = assembly.GetManifestResourceStream ("mono-ecma.css");
+				css_ecma = (new StreamReader (str_css)).ReadToEnd();
+			} else {
+				css_ecma = String.Empty;
+			}
+			return css_ecma;
+		}
+	}
 	public override string GetText (string url, out Node match_node)
 	{
 		match_node = null;
@@ -491,11 +508,14 @@ public class EcmaHelpSource : HelpSource {
 			XsltArgumentList args = new XsltArgumentList();
 			args.AddExtensionObject("monodoc://extensions", ExtObject);
 			args.AddParam("show", "", "masteroverview");
-			return Htmlize(new XPathDocument (summary), args);
+			string s = Htmlize(new XPathDocument (summary), args);
+			return BuildHtml (css_ecma_code, s); 
 		}
 		
-		if (url.StartsWith ("ecma:"))
-			return GetTextFromUrl (url);
+		if (url.StartsWith ("ecma:")) {
+			string s = GetTextFromUrl (url);
+			return BuildHtml (css_ecma_code, s); 
+		}
 
 		return null;
 	}
@@ -810,7 +830,8 @@ public class EcmaHelpSource : HelpSource {
 			args.AddExtensionObject("monodoc://extensions", ExtObject);
 			args.AddParam("show", "", "namespace");
 			args.AddParam("namespace", "", ns_name);
-			return Htmlize(new XPathDocument (summary), args);
+			string s = Htmlize(new XPathDocument (summary), args);
+			return BuildHtml (css_ecma_code, s); 
 
 		}
 		match_node = null;
@@ -907,7 +928,8 @@ public class EcmaHelpSource : HelpSource {
 		
 		if (rest == "") {
 			args.AddParam("show", "", "typeoverview");
-			return Htmlize(doc, args);
+			string s = Htmlize(doc, args);
+			return BuildHtml (css_ecma_code, s); 
 		}
 		
 		string [] nodes = rest.Split (new char [] {'/'});
@@ -963,7 +985,8 @@ public class EcmaHelpSource : HelpSource {
 			return "Unknown url: " + url;
 		}
 
-		return Htmlize(doc, args);
+		string html = Htmlize(doc, args);
+		return BuildHtml (css_ecma_code, html); 
 	}
 
 	
@@ -1006,7 +1029,11 @@ public class EcmaHelpSource : HelpSource {
 		if (ecma_transform == null) {
 			ecma_transform = new XslTransform ();
 			Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
-			Stream stream = assembly.GetManifestResourceStream ("mono-ecma.xsl");
+			Stream stream;
+			if (use_css) 
+				stream = assembly.GetManifestResourceStream ("mono-ecma-css.xsl");
+			else 
+				stream = assembly.GetManifestResourceStream ("mono-ecma.xsl");
 
 			XmlReader xml_reader = new XmlTextReader (stream);
 			ecma_transform.Load (xml_reader, null, null);
@@ -1154,6 +1181,9 @@ public class EcmaHelpSource : HelpSource {
 						return "Not implemented.";
 				}
 
+				// The following code is flakey and fails to find existing members
+				return "";
+#if false
 				MemberInfo[] mis = t.GetMember(membername, BF.Static | BF.Instance | BF.Public | BF.NonPublic);
 
 				if (mis.Length == 0)
@@ -1183,7 +1213,7 @@ public class EcmaHelpSource : HelpSource {
 
 					return MonoImpInfo(mi, "member", strlong);
 				}
-
+#endif
 				return "This member has not been implemented.";
 
 			} catch (Exception e) {
@@ -1254,6 +1284,10 @@ public class EcmaHelpSource : HelpSource {
 		{
 			return SettingsHandler.Settings.EnableEditing && Settings.RunningGUI;
 		}
+		
+		public bool IsToBeAdded(string text) {
+			return text.StartsWith("To be added");
+		}
 	}
 
 	//
@@ -1299,6 +1333,54 @@ public class EcmaHelpSource : HelpSource {
 		return null;
 	}
 	
+	//
+	// Obtain an URL of the type T:System.Object from the node
+	// 
+	public static string GetNiceUrl (Node node) {
+		if (node.Element.StartsWith("N:"))
+			return node.Element;
+		string name, full;
+		int bk_pos = node.Caption.IndexOf (' ');
+		// node from an overview
+		if (bk_pos != -1) {
+			name = node.Caption.Substring (0, bk_pos);
+			full = node.Parent.Caption + "." + name.Replace ('.', '+');
+			return "T:" + full;
+		}
+		// node that lists constructors, methods, fields, ...
+		if ((node.Caption == "Constructors") || (node.Caption == "Fields") || (node.Caption == "Events") 
+			|| (node.Caption == "Members") || (node.Caption == "Properties") || (node.Caption == "Methods")
+			|| (node.Caption == "Operators")) {
+			bk_pos = node.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			return "T:" + full + "/" + node.Element; 
+		}
+		int pr_pos = node.Caption.IndexOf ('(');
+		// node from a constructor
+		if (node.Parent.Element == "C") {
+			name = node.Parent.Parent.Parent.Caption;
+			int idx = node.URL.IndexOf ('/');
+			return node.URL[idx+1] + ":" + name + "." + node.Caption.Replace ('.', '+');
+		// node from a method with one signature, field, property, operator
+		} else if (pr_pos == -1) {
+			bk_pos = node.Parent.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			int idx = node.URL.IndexOf ('/');
+			return node.URL[idx+1] + ":" + full + "." + node.Caption;
+		// node from a method with several signatures
+		} else {
+			bk_pos = node.Parent.Parent.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Parent.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			int idx = node.URL.IndexOf ('/');
+			return node.URL[idx+1] + ":" + full + "." + node.Caption;
+		}
+		return "";
+
+	}
+				
 	//
 	// Populates the index.
 	//
@@ -1433,6 +1515,205 @@ public class EcmaHelpSource : HelpSource {
 			}
 		}
 	}
+	//
+	// Create list of documents for searching
+	//
+	public override void PopulateSearchableIndex (IndexWriter writer)
+	{
+		StringBuilder text;
+		foreach (Node ns_node in Tree.Nodes) {
+			Console.WriteLine ("\tNamespace: {0} ({1})", ns_node.Caption, ns_node.Nodes.Count);
+			foreach (Node type_node in ns_node.Nodes) {
+				string typename = type_node.Caption.Substring (0, type_node.Caption.IndexOf (' '));
+				string full = ns_node.Caption + "." + typename;
+				string doc_tag = GetKindFromCaption (type_node.Caption);
+				string url = "T:" + full;
+				string rest;
+				XmlDocument xdoc = GetXmlFromUrl (type_node.URL, out rest);
+				if (xdoc == null)
+					continue;
+				
+				// 
+				// For classes, structures or interfaces add a doc for the overview and
+				// add a doc for every constructor, method, event, ...
+				// 
+				if (doc_tag == "Class" || doc_tag == "Structure" || doc_tag == "Interface"){
+					
+					// Adds a doc for every overview of every type
+					SearchableDocument doc = new SearchableDocument ();
+					doc.title = type_node.Caption;
+					doc.hottext = typename;
+					doc.url = url;
+					
+					XmlNode node_sel = xdoc.SelectSingleNode ("/Type/Docs");
+					text  = new StringBuilder ();
+					GetTextFromNode (node_sel, text);
+					doc.text = text.ToString ();
+
+					text  = new StringBuilder ();
+					GetExamples (node_sel, text);
+					doc.examples = text.ToString ();
+					
+					writer.AddDocument (doc.LuceneDoc);
+
+					//Add docs for contructors, methods, etc.
+					foreach (Node c in type_node.Nodes) { // c = Constructors || Fields || Events || Properties || Methods || Operators
+						
+						if (c.Element == "*")
+							continue;
+						int i = 1;
+						foreach (Node nc in c.Nodes) {
+							//xpath to the docs xml node
+							string xpath;
+							if (c.Caption == "Constructors")
+								xpath = String.Format ("/Type/Members/Member[{0}]/Docs", i++);
+							else if (c.Caption == "Operators")
+								xpath = String.Format ("/Type/Members/Member[@MemberName='op_{0}']/Docs", nc.Caption);
+							else
+								xpath = String.Format ("/Type/Members/Member[@MemberName='{0}']/Docs", nc.Caption);
+							//construct url of the form M:Array.Sort
+							string urlnc;
+							if (c.Caption == "Constructors")
+								urlnc = String.Format ("{0}:{1}.{2}", c.Caption[0], ns_node.Caption, nc.Caption);
+							else
+								urlnc = String.Format ("{0}:{1}.{2}.{3}", c.Caption[0], ns_node.Caption, typename, nc.Caption);
+
+							//create the doc
+							SearchableDocument doc_nod = new SearchableDocument ();
+							doc_nod.title = LargeName (nc);
+							//dont add the parameters to the hottext
+							int ppos = nc.Caption.IndexOf ('(');
+							if (ppos != -1)
+								doc_nod.hottext =  nc.Caption.Substring (0, ppos);
+							else
+								doc_nod.hottext = nc.Caption;
+
+							doc_nod.url = urlnc;
+
+							XmlNode xmln = xdoc.SelectSingleNode (xpath);
+							if (xmln == null) {
+								Console.WriteLine ("Problem: {0}, with xpath: {1}", urlnc, xpath);
+								continue;
+							}
+
+							text = new StringBuilder ();
+							GetTextFromNode (xmln, text);
+							doc_nod.text = text.ToString ();
+
+							text = new StringBuilder ();
+							GetExamples (xmln, text);
+							doc_nod.examples = text.ToString ();
+
+							writer.AddDocument (doc_nod.LuceneDoc);
+						}
+					}
+				//
+				// Enumerations: add the enumeration values
+				//
+				} else if (doc_tag == "Enumeration"){
+										
+					XmlNodeList members = xdoc.SelectNodes ("/Type/Members/Member");
+					if (members == null)
+						continue;
+
+					text = new StringBuilder ();
+					foreach (XmlNode member_node in members) {
+						string enum_value = member_node.Attributes ["MemberName"].InnerText;
+						text.Append (enum_value);
+						text.Append (" ");
+						GetTextFromNode (member_node["Docs"], text);
+						text.Append ("\n");
+					}
+					SearchableDocument doc = new SearchableDocument ();
+
+					text = new StringBuilder ();
+					GetExamples (xdoc.SelectSingleNode ("/Type/Docs"), text);
+					doc.examples = text.ToString ();
+
+					doc.title = type_node.Caption;
+					doc.hottext = xdoc.DocumentElement.Attributes["Name"].Value;
+					doc.url = url;
+					doc.text = text.ToString();
+					writer.AddDocument (doc.LuceneDoc);
+				//
+				// Add delegates
+				//
+				} else if (doc_tag == "Delegate"){
+					SearchableDocument doc = new SearchableDocument ();
+					doc.title = type_node.Caption;
+					doc.hottext = xdoc.DocumentElement.Attributes["Name"].Value;
+					doc.url = url; 
+					
+					XmlNode node_sel = xdoc.SelectSingleNode ("/Type/Docs");
+
+					text = new StringBuilder ();
+					GetTextFromNode (node_sel, text);
+					doc.text = text.ToString();
+
+					text = new StringBuilder ();
+					GetExamples (node_sel, text);
+					doc.examples = text.ToString();
+
+					writer.AddDocument (doc.LuceneDoc);
+				} 
+			}
+		}
+	}
+	
+	//
+	// Extract the interesting text from the docs node
+	//
+	void GetTextFromNode (XmlNode n, StringBuilder sb) 
+	{
+		//don't include example code
+		if (n.Name == "code") 
+			return;
+
+		//include the url to which points the see tag
+		if (n.Name == "see" && n.Attributes.Count > 0)
+				sb.Append (n.Attributes [0].Value);
+		
+		//include the name of the parameter
+		if (n.Name == "paramref" && n.Attributes.Count > 0)
+			sb.Append (n.Attributes [0].Value);
+
+		//include the contents for the node that contains text
+		if (n.NodeType == XmlNodeType.Text)
+			sb.Append (n.Value);
+		
+		//add the rest of xml tags recursively
+		if (n.HasChildNodes)
+			foreach (XmlNode n_child in n.ChildNodes)
+				GetTextFromNode (n_child, sb);
+	}
+	//
+	// Extract the code nodes from the docs
+	//
+	void GetExamples (XmlNode n, StringBuilder sb)
+	{
+		if (n.Name == "code") {
+			sb.Append (n.InnerText);
+		} else {
+			if (n.HasChildNodes)
+				foreach (XmlNode n_child in n.ChildNodes)
+					GetExamples (n_child, sb);
+		}
+	}
+	//
+	// Extract a large name for the Node
+	//  (copied from mono-tools/docbrowser/browser.Render()
+	static string LargeName (Node matched_node)
+	{
+		string[] parts = matched_node.URL.Split('/', '#');			
+		if(parts.Length == 3 && parts[2] != String.Empty) { //List of Members, properties, events, ...
+			return parts[1] + ": " + matched_node.Caption;
+		} else if(parts.Length >= 4) { //Showing a concrete Member, property, ...					
+			return parts[1] + "." + matched_node.Caption;
+		} else {
+			return matched_node.Caption;
+		}
+	}
+
 }
 
 public class EcmaUncompiledHelpSource : EcmaHelpSource {
@@ -1451,7 +1732,7 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 		basedoc = new XmlDocument();
 		basedoc.Load(Path.Combine(basedir.FullName, "index.xml"));
 		
-		Name = basedoc.SelectSingleNode("Overview/Assembly").InnerText;
+		Name = basedoc.SelectSingleNode("Overview/Title").InnerText;
 		
 		bool has_content = false;
 		
