@@ -80,7 +80,7 @@ namespace Monodoc {
 				
 		}
 		
-		public static void SaveChange (string url, RootTree tree, XmlNode node)
+		public static void SaveChange (string url, RootTree tree, XmlNode node, string node_url)
 		{
 			string [] uSplit = ParseEditUrl (url);
 		
@@ -91,7 +91,7 @@ namespace Monodoc {
 				int prov = int.Parse (uSplit [0].Substring("monodoc://".Length));
 				HelpSource hs = tree.GetHelpSourceFromId (prov);
 				
-				changes.AddChange (hs.Name, hs.GetRealPath (id), xp, node);
+				changes.AddChange (hs.Name, hs.GetRealPath (id), xp, node, node_url);
 				changes.Save ();
 			} else if (uSplit[0].StartsWith("file:")) {
 				uSplit[0] = uSplit[0].Substring(5);
@@ -108,6 +108,24 @@ namespace Monodoc {
 				throw new NotImplementedException("Don't know how to save to " + url); 
 			}
 		}
+
+		public static void RemoveChange (string url, RootTree tree)
+		{
+			string [] uSplit = ParseEditUrl (url);
+		
+			string xp = uSplit [2];
+			string id = uSplit [1];
+						
+			if (uSplit[0].StartsWith("monodoc://")) {
+				int prov = int.Parse (uSplit [0].Substring("monodoc://".Length));
+				HelpSource hs = tree.GetHelpSourceFromId (prov);
+				
+				changes.RemoveChange (hs.Name, hs.GetRealPath (id), xp);
+				changes.Save ();
+			} else if (uSplit[0].StartsWith("file:")) {
+				//TODO: Not implemented
+			}
+		}
 		
 		public static void RenderEditPreview (string url, RootTree tree, XmlNode new_node, XmlWriter w)
 		{
@@ -265,11 +283,12 @@ namespace Monodoc {
 			}
 		}
 
-		Change NewChange (string xpath, XmlNode new_node)
+		Change NewChange (string xpath, XmlNode new_node, string node_url)
 		{
 			Change new_change = new Change ();
 			new_change.XPath = xpath;
 			new_change.NewNode = new_node;
+			new_change.NodeUrl = node_url;
 
 			Console.WriteLine ("New serial:" + SettingsHandler.Settings.SerialNumber);
 			new_change.Serial = SettingsHandler.Settings.SerialNumber;
@@ -277,10 +296,10 @@ namespace Monodoc {
 			return new_change;
 		}
 		
-		public void AddChange (string doc_set, string real_file, string xpath, XmlNode new_node)
+		public void AddChange (string doc_set, string real_file, string xpath, XmlNode new_node, string node_url)
 		{
 			FileChangeset new_file_change_set;
-			Change new_change = NewChange (xpath, new_node);
+			Change new_change = NewChange (xpath, new_node, node_url);
 			
 			if (real_file == null)
 				throw new Exception ("Could not find real_file. Please talk to Miguel or Ben about this");
@@ -325,6 +344,37 @@ namespace Monodoc {
 			DocSetChangesets.Add (new_dcs);
 		}
 
+		public void RemoveChange (string doc_set, string real_file, string xpath)
+		{
+			if (real_file == null)
+				throw new Exception ("Could not find real_file. Please talk to Miguel or Ben about this");
+			
+			for (int i = 0; i < DocSetChangesets.Count; i++) {
+				DocSetChangeset dscs = DocSetChangesets [i] as DocSetChangeset;
+				if (dscs.DocSet != doc_set) 
+					continue;
+
+				for (int j = 0; j < dscs.FileChangesets.Count; j++) {
+					FileChangeset fcs = dscs.FileChangesets [j] as FileChangeset;
+					if (fcs.RealFile != real_file)
+						continue;
+
+					for (int k = 0; k < fcs.Changes.Count; k++) {
+						Change c = fcs.Changes [k] as Change;
+						if (c.XPath == xpath) {
+							fcs.Changes.Remove (c);
+							break;
+						}
+					}
+					if (fcs.Changes.Count == 0)
+						dscs.FileChangesets.Remove (fcs);
+				}
+
+				if (dscs.FileChangesets.Count == 0)
+					DocSetChangesets.Remove (dscs);
+			}
+		}
+
 		public GlobalChangeset GetFrom (int starting_serial_id)
 		{
 			GlobalChangeset s = null;
@@ -391,6 +441,7 @@ namespace Monodoc {
 	public class Change {
 		[XmlAttribute] public string XPath;
 		[XmlAttribute] public int FromVersion = RootTree.MonodocVersion;
+		[XmlAttribute] public string NodeUrl;
 		
 		public XmlNode NewNode;
 
