@@ -21,6 +21,8 @@ using System.Xml;
 using System.Xml.XPath;
 using ICSharpCode.SharpZipLib.Zip;
 
+using Monodoc.Lucene.Net.Index;
+using Monodoc.Lucene.Net.Analysis.Standard;
 /// <summary>
 ///    This tree is populated by the documentation providers, or populated
 ///    from a binary encoding of the tree.  The format of the tree is designed
@@ -334,6 +336,9 @@ public class Node : IComparable {
 
 	public string URL {
 		get {
+			if (position < 0)
+				LoadNode ();
+
 			if (element.IndexOf (":") >= 0)
 				return element;
 
@@ -362,6 +367,23 @@ public class Node : IComparable {
 //
 public class HelpSource {
 	static int id;
+	public static bool use_css = false;
+	public static string css_code;
+	public static string CssCode {
+		get {
+			if (css_code != null)
+				return css_code;
+
+			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
+			Stream str_css = assembly.GetManifestResourceStream ("base.css");
+			StringBuilder sb = new StringBuilder ((new StreamReader (str_css)).ReadToEnd());
+			sb.Replace ("@@FONT_FAMILY@@", SettingsHandler.Settings.preferred_font_family);
+			sb.Replace ("@@FONT_SIZE@@", SettingsHandler.Settings.preferred_font_size.ToString());
+			css_code = sb.ToString ();
+			return css_code;
+		}
+		set { css_code = value; }
+	}
 
 	//
 	// The unique ID for this HelpSource.
@@ -631,6 +653,31 @@ public class HelpSource {
 	public virtual void PopulateIndex (IndexMaker index_maker)
 	{
 	}
+	
+	//
+	// Build an html document
+	//
+	public static string BuildHtml (string css, string html_code) {
+		StringWriter output = new StringWriter ();
+		output.Write ("<html><head>");
+		output.Write ("<style type=\"text/css\">");
+		output.Write (CssCode);
+		output.Write (css);
+		output.Write ("</style>");
+		output.Write ("</head><body>");
+		output.Write (html_code);
+		output.Write ("</body></html>");
+		return output.ToString ();
+	}
+
+	//
+	// Create different Documents for adding to Lucene search index
+	// The default action is do nothing. Subclasses should add the docs
+	// 
+	public virtual void PopulateSearchableIndex (IndexWriter writer) {
+		return;
+	}
+
 }
 
 public abstract class Provider {
@@ -658,7 +705,7 @@ public abstract class Provider {
 public class RootTree : Tree {
 	string basedir;
 	
-	public static Hashtable ExtraHelpSources = new Hashtable();
+	public static ArrayList UncompiledHelpSources = new ArrayList();
 	
 	public const int MonodocVersion = 1;
 	
@@ -845,10 +892,10 @@ public class RootTree : Tree {
 			}
 		}
 		
-		foreach (string key in ExtraHelpSources.Keys) {
-			HelpSource hs = (HelpSource)ExtraHelpSources[key];
+		foreach (string path in UncompiledHelpSources) {
+			EcmaUncompiledHelpSource hs = new EcmaUncompiledHelpSource(path);
 			root.help_sources.Add (hs);
-			Node hsn = root.CreateNode(key, "extra-help-source-" + key);
+			Node hsn = root.CreateNode(hs.Name, "extra-help-source-" + hs.Name);
 			hsn.EnsureNodes ();
 			foreach (Node n in hs.Tree.Nodes){
 				hsn.AddNode (n);
@@ -1050,6 +1097,7 @@ public class RootTree : Tree {
 		return (HelpSource) help_sources [id];
 	}
 	
+	string home_cache;
 	/// <summary>
 	///    Allows every HelpSource to try to provide the content for this
 	///    URL.
@@ -1059,12 +1107,82 @@ public class RootTree : Tree {
 		lastHelpSourceTime = DateTime.MinValue;
 		if (url == "root:") {
 			match_node = this;
-			StringBuilder sb = new StringBuilder ("<table bgcolor=\"#b0c4de\" width=\"100%\" cellpadding=\"5\"><tr><td><h3>Mono Documentation Library</h3></td></tr></table>");
+
+			// look whether there are contribs
+			GlobalChangeset chgs = EditingUtils.changes;
+			StringBuilder con = new StringBuilder ();
 			
-			foreach (Node n in Nodes)
-				sb.AppendFormat ("<a href='{0}'>{1}</a><br/>", n.Element, n.Caption);
+			//add links to the contrib
+			int oldContrib = 0, contribs = 0;
+			con.Append ("<ul>");
+			foreach (DocSetChangeset dscs in chgs.DocSetChangesets) 
+				foreach (FileChangeset fcs in dscs.FileChangesets) 
+					foreach (Change c in fcs.Changes) {
+						if (c.NodeUrl == null) {
+							if (c.Serial == SettingsHandler.Settings.SerialNumber)
+								oldContrib++;
+						} else if (c.Serial == SettingsHandler.Settings.SerialNumber) {
+							contribs++;
+							con.Append (String.Format ("<li><a href=\"{0}\">{0}</a></li>", c.NodeUrl));
+						}
+					}
 			
-			return sb.ToString ();	
+			string contrib = (oldContrib + contribs) == 1?"There is {0} contribution":"There are {0} contributions";
+			con.Insert (0, String.Format (contrib, oldContrib + contribs) + " for uploading <i>(File--&gt; Contributions--&gt; Upload)</i>", 1);
+			con.Append ("</ul>");
+			if (oldContrib == 1)
+				con.Append ("<i>You have 1 contribution that is not listed below and will be send the next time you upload contributions. Only contributions made from now on will be listed.</i>");
+			else if (oldContrib > 1)
+				con.Append ("<i>You have " + oldContrib + "contributions that are not listed below and will be send the next time you upload contributions. Only contributions made from now on will be listed.</i>");
+
+			//start the rendering
+			if (!HelpSource.use_css) {
+				StringBuilder sb = new StringBuilder ("<table bgcolor=\"#b0c4de\" width=\"100%\" cellpadding=\"5\"><tr><td><h3>Mono Documentation Library</h3></td></tr></table>");
+			
+				foreach (Node n in Nodes)
+					sb.AppendFormat ("<a href='{0}'>{1}</a><br/>", n.Element, n.Caption);
+			
+				//contributions
+				sb.Append ("<br><table bgcolor=\"#fff3f3\" width=\"100%\" cellpadding=\"5\"><tr><td>");
+				sb.Append ("<h5>Contributions</h5><br>");
+				if ((oldContrib + contribs) == 0) {
+					sb.Append ("<p><b>You don't have any contribution yet.</b></p>");
+					sb.Append ("<p>The Documentation of the libraries is not complete and your contributions would be greatly apreciated. The procedure is easy, browse to the part of the documentation you want to contribute and click on the <font color=\"blue\">[Edit]</font> link to start writing the documentation.</p>");
+					sb.Append ("<p>When you are happy with your changes, use the menu File--&gt; Contributions--&gt; Upload to upload your contributions to our server.</p></div>");
+				} else {
+					sb.Append (con.ToString ());
+				}
+				sb.Append ("</td></tr></table>");
+				return sb.ToString ();	
+			} else {
+				if (home_cache == null) {
+					System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly (typeof (HelpSource));
+					Stream hp_stream = assembly.GetManifestResourceStream ("home.html");
+					home_cache = (new StreamReader (hp_stream)).ReadToEnd ();
+				}
+				StringBuilder sb = new StringBuilder (home_cache);
+				// adjust fonts
+				sb.Replace ("@@FONT_FAMILY@@", SettingsHandler.Settings.preferred_font_family);
+				sb.Replace ("@@FONT_SIZE@@", SettingsHandler.Settings.preferred_font_size.ToString());
+				//contributions
+				if ((oldContrib + contribs) == 0) {
+					sb.Replace ("@@CONTRIB_DISP@@", "display: none;");
+				} else {
+					sb.Replace ("@@NO_CONTRIB_DISP@@", "display: none;");
+					sb.Replace ("@@CONTRIBS@@", con.ToString ());
+				}
+					
+				// load the url of nodes
+				String add_str;
+				StringBuilder urls = new StringBuilder ();
+				foreach (Node n in Nodes) {
+					add_str = String.Format ("<li><a href=\"{0}\">{1}</a></li>", n.Element, n.Caption);
+					urls.Append (add_str);
+				}
+				sb.Replace ("@@API_DOCS@@", urls.ToString ());
+						
+				return sb.ToString ();
+			}
 		} 
 		
 		if (url.StartsWith ("root:")) {
@@ -1165,6 +1283,45 @@ public class RootTree : Tree {
 		Console.WriteLine ("Documentation index updated");
 	}
 	
+	// Search Index
+	public SearchableIndex GetSearchIndex ()
+	{
+		return SearchableIndex.Load (Path.Combine (basedir, "search_index"));
+	}
+
+	public static void MakeSearchIndex ()
+	{
+		// Loads the RootTree
+		Console.WriteLine ("Loading the monodoc tree...");
+		RootTree root = LoadTree ();
+		if (root == null)
+			return;
+
+		string dir = Path.Combine (root.basedir, "search_index");
+		IndexWriter writer;
+		//try to create the dir to store the index
+		try {
+			if (!Directory.Exists (dir)) 
+				Directory.CreateDirectory (dir);
+
+			writer = new IndexWriter(Lucene.Net.Store.FSDirectory.GetDirectory(dir, true), new StandardAnalyzer(), true);
+		} catch (UnauthorizedAccessException) {
+			Console.WriteLine ("You don't have permissions to wirte on " + dir);
+			return;
+		}
+
+		//Collect all the documents
+		Console.WriteLine ("Collecting and adding documents...");
+		foreach (HelpSource hs in root.HelpSources) 
+			hs.PopulateSearchableIndex (writer);
+	
+		//Optimize and close
+		Console.WriteLine ("Closing...");
+		writer.Optimize();
+		writer.Close();
+	}
+
+
 	public ICollection HelpSources { get { return new ArrayList(help_sources); } }
 
 	[System.Runtime.InteropServices.DllImport ("libc")]
